@@ -320,15 +320,12 @@ int submit_result() {
     gm_log( GM_LOG_TRACE, "queue: %s\n", mod_gm_opt->result_queue );
     resultsize = sizeof(char) * strlen(mod_gm_opt->message) + GM_BUFFERSIZE;
     result = gm_malloc(resultsize);
-    snprintf( result, resultsize-1, "type=%s\nhost_name=%s\nstart_time=%i.%i\nfinish_time=%i.%i\nlatency=%i.%i\nreturn_code=%i\nsource=send_gearman\n",
+    snprintf( result, resultsize-1, "type=%s\nhost_name=%s\nstart_time=%Lf\nfinish_time=%Lf\nlatency=%Lf\nreturn_code=%i\nsource=send_gearman\n",
               mod_gm_opt->active == GM_ENABLED ? "active" : "passive",
               mod_gm_opt->host,
-              (int)starttime.tv_sec,
-              (int)starttime.tv_usec,
-              (int)finishtime.tv_sec,
-              (int)finishtime.tv_usec,
-              ( int )mod_gm_opt->latency.tv_sec,
-              ( int )mod_gm_opt->latency.tv_usec,
+              timeval2double(&starttime),
+              timeval2double(&finishtime),
+              timeval2double(&mod_gm_opt->latency),
               mod_gm_opt->return_code
             );
 
