@@ -879,16 +879,16 @@ void print_version() {
 
 
 /* return human readable name for neb type */
-char * nebtype2str(int i) {
+char * nebtype2str(__attribute__((__unused__)) int i) {
     return gm_strdup("UNKNOWN");
 }
 
 /* return human readable name for nebcallback */
-char * nebcallback2str(int i) {
+char * nebcallback2str(__attribute__((__unused__)) int i) {
     return gm_strdup("UNKNOWN");
 }
 
 /* return human readable name for eventtype */
-char * eventtype2str(int i) {
+char * eventtype2str(__attribute__((__unused__)) int i) {
     return gm_strdup("UNKNOWN");
 }
